<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Observer;

use FiloBlu\ExtAmastyGeoipRedirect\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 *
 */
class RedirectResponseBefore implements ObserverInterface
{
    /**
     * @var \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data
     */
    protected $helper;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\UrlInterface $urlBuilder
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        UrlInterface $urlBuilder,
        StoreManagerInterface $storeManager,
        Data $helper
    ) {
        $this->helper = $helper;
        $this->scopeConfig = $scopeConfig;
        $this->urlBuilder = $urlBuilder;
        $this->storeManager = $storeManager;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function execute(Observer $observer)
    {
        $request = $observer->getRequest();
        $response = $observer->getResponse();

        $currentStoreId = $this->storeManager->getStore()->getId();

        $redirectAllowed = $this->helper->applyLogic($request, false, $this->urlBuilder->getCurrentUrl(), null);
        $enabled = $this->helper->isEnabled($currentStoreId);
        // Set header to prevent varnish caching
        //            $response->setHeader('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0', true);
        //            $response->setHeader('Pragma', 'no-cache', true);
        if ($enabled && $redirectAllowed && !$this->helper->geolocationRequest($request)) {
            $head = '<script>window.filoblu_geoip_redirect_allowed = true;</script>';
            $response->setBody(preg_replace('/<\/head>/', $head . '</head>', $response->getBody(), 1));
        }
    }
}
